'use strict';

const { exec, spawn } = require('child_process');
const os = require('os');
const path = require('path');
const fs = require('fs');

module.exports = async (opt) => {
  opt = opt || {};
  let {
    id,
    output,
    debug = false,
    port,
    daemon = false,
    autoUpdate = true
  } = opt;

  let ver = fs.readFileSync(path.resolve(__dirname, '../.mongodal_ver'), 'utf8');
  let procCmd = path.resolve(__dirname, `../bin/mongo-dal@${ver}`);

  await killProc(`mongo-dal`);
  await new Promise((resolve, reject) => {    
    function initResolveCb() {
      resolve();
      process.removeListener('SIGURG', initResolveCb);
    };
    process
      .on('SIGURG', initResolveCb)
      .on('exit', clear(!daemon))
      .on('SIGINT', clear(!daemon))
      .on('SIGTERM', clear(!daemon));

    let args = ['-id', id];

    if (debug) {
      args.push('-d');
    }

    if (port != undefined) {
      port = port || 30017;
      args.push('-p', port);
    }

    let opts = undefined
    if (daemon) {
      args.push('-D');
      opts = { detached: true };
      if (!output) {
        args.push('-o', 'mongo-dal.log');
      }
    } else {
      if (typeof output === 'string' && output.length) {
        args.push('-o', output);
      } else {
        opts = { stdio: ['ignore', 'inherit', 'inherit'] };
      }
    }

    if (autoUpdate) {
      args.push('-u')
    }

    let subProcess = spawn(procCmd, args, opts);
    if (daemon) {
      subProcess.unref();
    }
  })
};

function killProc(name) {
  return new Promise((resolve, reject) => {
    exec(`pgrep ${name}`, (err, stdout, stderr) => {
      if (err) {
        return resolve()
      }
      let pidsArg = stdout.replace(/\s/, ' ')

      exec(`kill ${pidsArg}`, (err, stdout, stderr) => {
        resolve(stdout);
      });
    });
  });
}

function _getEnv(opt) {
  let env = process.env.NODE_ENV || opt.env;

  switch(env) {
    case 'alpha':
    case 'alta':
    case 'altb':
    case 'prod':
      return env;
    case 'production':
      return 'prod';
    default:
      return 'alpha';
  }
}

function clear(needRemove) {
  return () => {
    if (needRemove) {
      const MONGODAL_PID = fs.readFileSync('/tmp/mongo_dal_pid', 'utf8');
      console.log("mongo_dal_pid: ", MONGODAL_PID);
      process.kill(Number(MONGODAL_PID), 'SIGTERM');
    }
    console.log('clear success');
    process.exit(0);
  };
}
